/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */

using System;
using System.Collections;
using System.IO;

namespace TopCoder.File.Statistics.Processor.LineCounter
{
    /// <summary>
    /// A state machine framework based on state delegates, intended for line counting. The class does NOT rely on
    /// reading full lines, which may be inefficient in pathological files. (Files that were inadvertently classified
    /// as text files, while really being binaries with very long "lines".) It's used for line counting both in general
    /// text files and in C style source.
    /// </summary>
    internal abstract class SimpleStateMachine
    {
        /// <summary>
        /// This is the delegate for every state handler in the machine.
        /// </summary>
        /// <param name="nextChar">The next character, or -1, if EOF has been reached.</param>
        /// <returns>The state handler for the next character.</returns>
        public delegate StateDelegate StateDelegate(int nextChar);

        /// <summary>
        /// This method is always used as the start state. Abstract in the framework class.
        /// </summary>
        /// <param name="nextChar">The next character, or -1, if EOF has been reached.</param>
        /// <returns>The state handler for the next character.</returns>
        public abstract StateDelegate StateStart(int nextChar);

        /// <summary>
        /// The number of lines in the file, increased in relevant state handlers.
        /// </summary>
        protected long lines = 0;

        /// <summary>
        /// Exposes the internal line counter.
        /// </summary>
        /// <value>
        /// <c>lines</c>
        /// </value>
        public long Lines
        {
            get
            {
                return lines;
            }
        }

        /// <summary>
        /// A delegate representing the current state.
        /// </summary>
        private StateDelegate currentState;

        /// <summary>
        /// Default constructor, initializes current state.
        /// </summary>
        public SimpleStateMachine()
        {
            currentState = new StateDelegate(StateStart);
        }

        /// <summary>
        /// Opens a file for text access and feeds it into the state machine, adding an EOF in the end.
        /// </summary>
        /// <param name="file">The file to process.</param>
        /// <exception cref="ArgumentNullException">If <c>file</c> is null.</exception>
        /// <exception cref="IOException">For any IO error.</exception>
        public void Process(FileInfo file)
        {
            if (file == null)
            {
                throw new ArgumentNullException("file");
            }

            using (TextReader reader = file.OpenText())
            {
                int nowChar;
                do
                {
                    nowChar = reader.Read();
                    currentState = currentState(nowChar);
                } while (nowChar != -1);
            }
        }
    }
}