/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */

using System;
using System.Collections;
using System.IO;

namespace TopCoder.File.Statistics.Processor.LineCounter
{
    /// <summary>
    /// <c>TextLineCounter</c> counts lines for text files. It supports a single metric of "Line Count".
    /// </summary>
    /// <remarks>
    /// The configuration is equivalent to the base class, no differences (<see cref="TextLineCounter" />).
    /// The class is stateless and threadsafe.
    /// </remarks>
    public class TextLineCounter : FileProcessorBare
    {
        /// <summary>
        /// The name of the only supported metric.
        /// </summary>
        public const string Name = "Line Count";

        /// <summary>
        /// The display name (XML element name) of the only supported metric.
        /// </summary>
        public const string DisplayName = "line";

        /// <summary>
        /// Description of the only supported metric.
        /// </summary>
        private const string Description = "Line count (in a text file)";

        /// <summary>
        /// An immutable reference to the only supported metric.
        /// </summary>
        private static readonly Metric LineMetric = new Metric(Name, DisplayName, Description);

        /// <summary>
        /// Exposes a list of the supported metrics, in this case only one, line count.
        /// </summary>
        /// <value>
        /// A non-null <c>IList</c> with exactly one <c>Metric</c> element.
        /// </value>
        public override IList SupportedMetrics
        {
            get
            {
                IList list = new ArrayList();
                list.Add(LineMetric);

                return list;
            }
        }

        /// <summary>
        /// Constructor with both alias and matcher.
        /// </summary>
        /// <param name="alias">The alias for the file processor, non-empty, non-null.</param>
        /// <param name="matcher">The file matcher to use, non-null.</param>
        /// <exception cref="ArgumentNullException">If <c>alias</c> or <c>matcher</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>alias</c> is empty, when trimmed.</exception>
        public TextLineCounter(string alias, IFileMatcher matcher)
            : base(alias, matcher)
        {
        }

        /// <summary>
        /// This constructor loads configuration from a namespace and a property name prefix. The non-empty alias and
        /// non-null file matcher are required. Any error will be wrapped in a <c>ConfigurationException</c>.
        /// </summary>
        /// <param name="nameSpace">The configuration manager namespace.</param>
        /// <param name="prefix">The prefix to the specific property names.</param>
        /// <exception cref="ArgumentNullException">If <c>nameSpace</c> or <c>prefix</c> is null.</exception>
        /// <exception cref="ConfigurationException">Any configuration error.</exception>
        public TextLineCounter(string nameSpace, string prefix)
            : base(nameSpace, prefix)
        {
        }

        /// <summary>
        /// Opens the file, counts its lines by the means of a state machine and returns the result.
        /// </summary>
        /// <param name="file">The file to process.</param>
        /// <exception cref="ArgumentNullException">If <c>file</c> is null.</exception>
        /// <exception cref="IOException">For any IO error.</exception>
        /// <returns>A non-null <c>StatsResult</c> with a value for the supported metric.</returns>
        public override StatsResult Process(FileInfo file)
        {
            SimpleStateMachine machine = new TextCountStateMachine();
            machine.Process(file);

            StatsResult result = new StatsResult();
            result[LineMetric] = machine.Lines;

            return result;
        }

        /// <summary>
        /// A simple state machine collapsing all CRLF and LFCR to single lines when counted.
        /// <see cref="SimpleStateMachine" />
        /// </summary>
        private class TextCountStateMachine : SimpleStateMachine
        {
            /// <summary>
            /// A state method for handling carriage return.
            /// </summary>
            /// <param name="nextChar">The next character, as an int, where -1 is EOF.</param>
            /// <returns>The delegate for the state to use for the next character.</returns>
            public StateDelegate StateCR(int nextChar)
            {
                switch (nextChar)
                {
                    // The only difference is that CRLF is considered to be ONE newline, for every other case, we
                    // rely on the general method.
                    case '\n':
                        return new StateDelegate(StateLF);
                    default:
                        return StateStart(nextChar);
                }
            }

            /// <summary>
            /// A state method for handling line feed.
            /// </summary>
            /// <param name="nextChar">The next character, as an int, where -1 is EOF.</param>
            /// <returns>The delegate for the state to use for the next character.</returns>
            public StateDelegate StateLF(int nextChar)
            {
                switch (nextChar)
                {
                    // The only difference is that LFCR is considered to be ONE newline, for every other case, we
                    // rely on the general method.
                    case '\r':
                        return new StateDelegate(StateCR);
                    default:
                        return StateStart(nextChar);
                }
            }

            /// <summary>
            /// A state method for handling the normal character flow. CR, LF and EOF trigger line number increments.
            /// CR and LF affects the state for the next character to handle CRLF/LFCR.
            /// </summary>
            /// <param name="nextChar">The next character, as an int, where -1 is EOF.</param>
            /// <returns>The delegate for the state to use for the next character.</returns>
            public override StateDelegate StateStart(int nextChar)
            {
                switch (nextChar)
                {
                    case '\r':
                        lines++;
                        return new StateDelegate(StateCR);
                    case '\n':
                        lines++;
                        return new StateDelegate(StateLF);
                    case -1: // EOF
                        lines++; // No state change needed, this is the end
                        return new StateDelegate(StateStart);
                    default: // Just go on...
                        return new StateDelegate(StateStart);
                }
            }
        }
    }
}