/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */

using System;
using System.Collections;

namespace TopCoder.File.Statistics.Reporting
{
    /// <summary>
    /// <c>StatsCollection</c> is a generic container for mapping from String keys to StatsResult instances. It is used
    /// in <c>AbstractStatsReporting</c> to hold statistic mappings based on file, directory and alias.
    /// The class provides basic manipulation methods for the mapping.
    /// </summary>
    /// <remarks>
    /// This class is NOT threadsafe.
    /// </remarks>
    public class StatsCollection
    {
        /// <summary>
        /// This is the real storage that this class just wraps, thinly. String keys are used.
        /// </summary>
        private IDictionary stats = new Hashtable();

        /// <summary>
        /// This property, just a getter, exposes all mapped keys. It is just a copy.
        /// </summary>
        /// <value>
        /// <c>stats.Keys</c>, wrapped in an <c>IList</c>.
        /// </value>
        public IList AllKeys
        {
            get
            {
                return new ArrayList(stats.Keys);
            }
        }

        /// <summary>
        /// Retrieves a key from storage. The value returned is a clone of the internally stored value.
        /// </summary>
        /// <param name="key">The key to retrieve results for.</param>
        /// <exception cref="ArgumentNullException">If <c>key</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>key</c>, when trimmed, is empty.</exception>
        /// <returns>A clone of the internally stored <c>StatsResult</c>, null if no value is stored.</returns>
        public StatsResult this[string key]
        {
            get
            {
                if (key == null)
                {
                    throw new ArgumentNullException("key");
                }
                if (stats == null)
                {
                    throw new ArgumentNullException("stats");
                }

                if (key.Trim().Length == 0)
                {
                    throw new ArgumentException("Empty string passed.", "key");
                }

                StatsResult result = (StatsResult) stats[key];

                // If it exists, clone it. We do NEVER expose internally stored data directly.
                if (result != null)
                {
                    return new StatsResult(result);
                }
                else
                {
                    return null;
                }
            }
        }

        /// <summary>
        /// Default constructor for doing any initialization needed. Nothing done in this version.
        /// </summary>
        public StatsCollection()
        {
        }

        /// <summary>
        /// Add a result to specified key.
        /// </summary>
        /// <remarks>
        /// This is the point of having a specific class. If the key already exists, the results are aggregated,
        /// summed. If not, a clone is made. In no event is the <c>stats</c> object stored directly.
        /// </remarks>
        /// <param name="key">The key to use.</param>
        /// <param name="stats">The stats to add.</param>
        /// <exception cref="ArgumentNullException">If <c>key</c> or <c>stats</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>key</c>, when trimmed, is empty.</exception>
        public void Add(string key, StatsResult stats)
        {
            if (key == null)
            {
                throw new ArgumentNullException("key");
            }
            if (stats == null)
            {
                throw new ArgumentNullException("stats");
            }

            if (key.Trim().Length == 0)
            {
                throw new ArgumentException("Empty string passed.", "key");
            }

            // If it already exists...
            if (this.stats.Contains(key))
            {
                // aggregate
                ((StatsResult) this.stats[key]).Aggregate(stats);
            }
            else
            {
                // if not, clone the argument.
                this.stats[key] = new StatsResult(stats);
            }
        }

        /// <summary>
        /// Clear the collection.
        /// </summary>
        public void Clear()
        {
            stats.Clear();
        }
    }
}