/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */


using System;
using System.Collections;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// <c>StatsResult</c> is a container for multiple results, each of which are mapped from a <c>Metric</c> to a
    /// <c>long</c>. This class wraps some map operations as well as provides a way to clone and aggregate results.
    /// </summary>
    /// <remarks>
    /// This class is not thread-safe.
    /// </remarks>
    public class StatsResult
    {
        /// <summary>
        /// The results mapping from <c>Metric</c> to the statistical quantity (<c>long</c>). This reference is never
        /// changed.
        /// </summary>
        /// <remarks>
        /// The dictionary reference is never changed, while the content may be updated.
        /// </remarks>
        private readonly IDictionary results;

        /// <summary>
        /// Get a list of all the associated metrics.
        /// </summary>
        /// <value>
        /// An <c>IList</c> of <c>Metric</c> instances, reflecting the metrics with associated values.
        /// </value>
        public IList Metrics
        {
            get
            {
                return new ArrayList(results.Keys);
            }
        }

        /// <summary>
        /// Set or get a certain stat (associate metric with value). Any old value will be overwritten if setting.
        /// </summary>
        /// <param name="metric">The metric for which to get/set a value.</param>
        /// <exception cref="ArgumentNullException">If <c>metric</c> is null.</exception>
        /// <exception cref="ArgumentException">In getter, if no value is associated for <c>metric</c>.</exception>
        /// <value>A <c>long</c> for the specified metric.</value>
        public long this[Metric metric]
        {
            get
            {
                object value = results[metric];
                if (value == null)
                {
                    throw new ArgumentException("No value present for specified metric.");
                }

                return (long) value;
            }
            set
            {
                results[metric] = value;
            }
        }

        /// <summary>
        /// Default constructor. Initializes the internal mapping.
        /// </summary>
        public StatsResult()
        {
            results = new Hashtable();
        }

        /// <summary>
        /// Create a result based on another result. This acts as a way to clone the instance.
        /// </summary>
        /// <param name="result">The result to clone.</param>
        /// <exception cref="ArgumentNullException">If <c>result</c> is null.</exception>
        public StatsResult(StatsResult result)
        {
            if (result == null)
            {
                throw new ArgumentNullException("result");
            }

            // We could reuse the Aggregate implementation.
            // However, cloning is performed quite frequently in the component, and just copying a hashtable is a
            // simpler and more efficient operation.

            // For example, we avoid any hash table resizing needed, and the checking of existing elements.
            results = new Hashtable(result.results);
        }

        /// <summary>
        /// Aggregate the result into the current result. This means that non-existing metrics are inserted and
        /// existing metric values are increased with the values contained in <c>result</c>.
        /// </summary>
        /// <param name="result">The result to insert from.</param>
        /// <exception cref="ArgumentNullException">If <c>result</c> is null.</exception>
        public void Aggregate(StatsResult result)
        {
            if (result == null)
            {
                throw new ArgumentNullException("result");
            }

            foreach (DictionaryEntry entry in result.results)
            {
                if (results.Contains(entry.Key))
                {
                    // Key already exists, perform addition
                    results[entry.Key] = (long) results[entry.Key] + (long) entry.Value;
                }
                else
                {
                    // Just copy
                    this[(Metric) entry.Key] = (long) entry.Value;
                }
            }
        }
    }
}