/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0b
 */


using System;
using System.Collections;
using System.IO;
using System.Text.RegularExpressions;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// An implementation of <c>IFileMatcher</c>, based on <c>FileNameMatcher</c>. It allows checking of a file name
    /// against one or several regular expressions (where matching at least one of them is considered a matching name).
    /// </summary>
    /// <remarks>
    /// The reflection contract for instantiation of a <c>IFileMatcher</c> instance is supported. The expression
    /// strings are parsed from the subproperty "patterns" in this case. It can be multivalued, representing several
    /// regex strings.
    ///
    /// Please note that a regular expression here is NOT equivalent to "normal" wildcards, like "*.cs". In fact, that
    /// expression isn't even a valid regular expression. <seealso cref="Regex"/>
    ///
    /// The implementation is stateless, the list of regular expressions immutable and consequently thread-safe.
    /// </remarks>
    public class RegexMatcher : FileNameMatcher
    {
        /// <summary>
        /// The name of the sub-property in the configuration where the regex pattern strings should be listed.
        /// </summary>
        public const string PatternsPropertyKey = "patterns";

        /// <summary>
        /// An <c>IList</c> of the patterns used by this matcher, all values should be non-null <c>Regex</c> instances.
        /// Set in constructors and only changed there.
        /// </summary>
        private readonly IList patterns;

        /// <summary>
        /// Constructor for a single pattern. Simply creates a list to hold it.
        /// </summary>
        /// <param name="pattern">The pattern to match file names against.</param>
        /// <exception cref="ArgumentNullException">If <c>pattern</c> is null.</exception>
        public RegexMatcher(Regex pattern)
        {
            if (pattern == null)
            {
                throw new ArgumentNullException("pattern");
            }

            patterns = new ArrayList(1);
            patterns.Add(pattern);
        }

        /// <summary>
        /// Constructor for a list of one or several patterns. The passed-in list is validated and cloned.
        /// </summary>
        /// <param name="patterns">A list of one or more non-null <c>Regex</c> instances to validate against.</param>
        /// <exception cref="ArgumentNullException">If <c>patterns</c> is null.</exception>
        /// <exception cref="ArgumentException">If any element in <c>patterns</c> is not a non-null <c>Regex</c>
        /// instance.</exception>
        public RegexMatcher(IList patterns)
        {
            if (patterns == null)
            {
                throw new ArgumentNullException("patterns");
            }

            this.patterns = new ArrayList(patterns);
            if (this.patterns.Count == 0)
            {
                throw new ArgumentException("Empty patterns list specified.");
            }
            foreach (object value in this.patterns)
            {
                if (!(value is Regex))
                {
                    throw new ArgumentException("At least one element in patterns not non-null Regex.");
                }
            }
        }

        /// <summary>
        /// The contract constructor, instantiating an <c>IFileMatcher</c> based on regular expressions from a
        /// configuraiton manager property.
        /// </summary>
        /// <remarks>
        /// The patterns should be stored as values in the subproperty "patterns" to this property.
        /// The patterns are compiled for maximum performance.
        /// <see cref="PatternsPropertyKey" />
        /// </remarks>
        /// <param name="nameSpace">The configuration manager namespace to use.</param>
        /// <param name="prefix">The prefix to the specific configuration properties.</param>
        /// <exception cref="ArgumentNullException">If <c>nameSpace</c> or <c>prefix</c> is null.</exception>
        /// <exception cref="ArgumentException">If <c>nameSpace</c>, when trimmed, is empty.</exception>
        /// <exception cref="ConfigurationException">If a property or the namespace doesn't exist, or a property is
        /// empty, or if it contains invalid pattern strings.</exception>
        public RegexMatcher(string nameSpace, string prefix)
        {
            Helper.CheckConfigData(nameSpace, prefix);

            string[] patterns = Helper.ReadValues(nameSpace, prefix, PatternsPropertyKey, false);
            this.patterns = new ArrayList(patterns.Length);

            try
            {
                foreach (string pattern in patterns)
                {
                    // Add a compiled instance
                    this.patterns.Add(new Regex(pattern, RegexOptions.Compiled));
                }
            }
            catch (Exception e)
            {
                throw new ConfigurationException("Error instantiating regex.", e);
            }
        }

        /// <summary>
        /// Template method to match a filename.
        /// </summary>
        /// <param name="fileName">Non-null file name.</param>
        /// <exception cref="FileMatchingException">Any error condition.</exception>
        /// <returns></returns>
        protected override bool Match(string fileName)
        {
            if (fileName == null)
            {
                throw new ArgumentNullException("fileName");
            }
            if (fileName.Trim().Length == 0)
            {
                throw new ArgumentException("Empty file name specified.", "fileName");
            }

            foreach (Regex pattern in patterns)
            {
                if (pattern.IsMatch(fileName))
                {
                    return true;
                }
            }
            return false;
        }
    }
}
