///
/// $(#)TypeNameMatcherTests.java		Sep 6, 2005
/// 
/// Copyright (c) 2005, TopCoder Inc. All rights reserved.
/// </summary>

using System;
using System.IO;
using System.Collections;
using NUnit.Framework;

using TopCoder.File.Statistics.Matcher;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.FailureTests.Matcher
{
	/// <summary>
    /// This test fixture contains the failure tests for <see cref="BinaryFileMatcher"/> class.
    ///
    /// <author>MPhk</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    [TestFixture]
	public class TypeNameMatcherTests {
	
	    /// <summary>
	    /// The namespace for this test.
	    /// </summary>
	    private const string NAMESPACE = "TopCoder.File.Statistics.Matcher.FailureTests";
	
	    /// <summary>
	    /// The config file for this test.
	    /// </summary>
	    private const string CONFIG_FILE = "../../test_files/failuretests/matcher/config.xml";
	
	    /// <summary>
	    /// The namespace for Magic Numbers component.
	    /// </summary>
	    private const string MAGIC_NUMBERS_CONFIG = "../../test_files/failuretests/MagicNumbers.xml";
	
	    /// <summary>
	    /// The the base path of the test files.
	    /// </summary>
	    private const string BASE_PATH = "../../test_files/failuretests/matcher/";
	
	    /// <summary>
	    /// A TypeNameMatcher instance to test.
	    /// </summary>
	    private TypeNameMatcher matcher = null;
	
	    /// <summary>
	    /// Load the configuration before testing.
	    /// </summary>
	    [SetUp]
	    protected void SetUp() {
	        // clear the namespace first
	        ConfigManager configManager = ConfigManager.GetInstance();
	        configManager.RemoveFile(CONFIG_FILE);
	        // load the configuration
	        configManager.LoadFile(CONFIG_FILE);
	        configManager.RemoveFile(MAGIC_NUMBERS_CONFIG);
	        configManager.LoadFile(MAGIC_NUMBERS_CONFIG);
	        
	        matcher = new TypeNameMatcher("GIF Image");
	    }
	
	    /// <summary>
	    /// Remove the configuration after testing.
	    /// </summary>
	    [TearDown]
	    protected void TearDown() {
	        ConfigManager configManager = ConfigManager.GetInstance();
	        configManager.RemoveFile(CONFIG_FILE);
	        configManager.RemoveFile(MAGIC_NUMBERS_CONFIG);
	    }
	
	    /// <summary>
	    /// Checks whether the constructor TypeNameMatcher(Pattern) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestTypeNameMatcher1() {
            new TypeNameMatcher((String) null);
	    }
	    
	    /// <summary>
	    /// Checks whether the constructor TypeNameMatcher(Pattern) properly throws
	    /// ArgumentException for empty String argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestTypeNameMatcher2() {
            new TypeNameMatcher("");
	    }
	
	    /// <summary>
	    /// Checks whether the constructor TypeNameMatcher(string, string) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestTypeNameMatcher3() {
            new TypeNameMatcher(null, "prefix.");
	    }
	    
	    /// <summary>
	    /// Checks whether the constructor TypeNameMatcher(string, string) properly throws
	    /// ArgumentException for empty argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestTypeNameMatcher4() {
            new TypeNameMatcher("", "prefix.");
	    }
	    
	    /// <summary>
	    /// Checks whether the constructor TypeNameMatcher(string, string) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestTypeNameMatcher5() {
            new TypeNameMatcher("namespace", null);
	    }
	
	    /// <summary>
	    /// Checks whether the constructor TypeNameMatcher(Property) properly throws
	    /// Configuration for invalid property - no values in "mimes" sub-property.
	    /// </summary>
	    [Test, ExpectedException(typeof(ConfigurationException))]
	    public void TestTypeNameMatcher6() {
            new TypeNameMatcher(NAMESPACE, "types_invalid1.");
	    }
	
	    /// <summary>
	    /// Checks whether the constructor TypeNameMatcher(Property) properly throws
	    /// Configuration for invalid property - no "mimes" sub-property.
	    /// </summary>
	    [Test, ExpectedException(typeof(ConfigurationException))]
	    public void TestTypeNameMatcher7() {
            new TypeNameMatcher(NAMESPACE, "types_invalid2.");
	    }
	
	    /// <summary>
	    /// Checks whether the constructor TypeNameMatcher(Property) properly throws
	    /// Configuration for invalid property - empty values.
	    /// </summary>
	    [Test, ExpectedException(typeof(ConfigurationException))]
	    public void TestTypeNameMatcher8() {
            new TypeNameMatcher(NAMESPACE, "types_invalid3.");
	    }
	    
	    /// <summary>
	    /// Checks whether the constructor TypeNameMatcher(List) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestTypeNameMatcher9() {
            new TypeNameMatcher((IList) null);
	    }
	
	    /// <summary>
	    /// Checks whether the constructor TypeNameMatcher(List) properly throws
	    /// ArgumentException for empty List argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestTypeNameMatcher10() {
            new TypeNameMatcher(new ArrayList());
	    }
	
	    /// <summary>
	    /// Checks whether the constructor TypeNameMatcher(List) properly throws
	    /// ArgumentException for List argument containing nulls.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestTypeNameMatcher11() {
	        IList list = new ArrayList();
	        list.Add("1");
	        list.Add(null);
	
            new TypeNameMatcher(list);
	    }
	
	    /// <summary>
	    /// Checks whether the constructor TypeNameMatcher(List) properly throws
	    /// ArgumentException for List argument containing empty Strings.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestTypeNameMatcher12() {
	        IList list = new ArrayList();
	        list.Add("1");
	        list.Add("");
	
            new TypeNameMatcher(list);
	    }
	
	    /// <summary>
	    /// Checks whether the constructor TypeNameMatcher(List) properly throws
	    /// ArgumentException for List argument containing non String elements.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestTypeNameMatcher13() {
	        IList list = new ArrayList();
	        list.Add("1");
	        list.Add(new Object());
	
            new TypeNameMatcher(list);
	    }
	
	    /// <summary>
	    /// Tests the match method with null argument. ArgumentNullException should be thrown.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestMatch1() {
            matcher.Match((FileInfo) null);
	    }
	
	    /// <summary>
	    /// Tests the match method for inexistent file. IOException should be thrown.
	    /// </summary>
	    [Test, ExpectedException(typeof(IOException))]
	    public void TestMatch2() {
	    	try
	    	{
            	matcher.Match(new FileInfo(BASE_PATH + "/inexistent_file"));
	    	}
	    	catch (IOException)
	    	{
	    		throw new IOException();
	    	}
	    }
	
	    /// <summary>
	    /// Tests the match method for a folder. IOException should be thrown.
	    /// </summary>
	    [Test, ExpectedException(typeof(IOException))]
	    public void TestMatch3() {
	    	try
	    	{
	        	matcher.Match(new FileInfo(BASE_PATH));
	    	} 
	    	catch (IOException)
	    	{
	    		throw new IOException();
	    	}
	    }
    }
}
