///
/// $(#)BasicStatsReporting.cs		Sep 6, 2005
/// 
/// Copyright (c) 2005, TopCoder Inc. All rights reserved.
/// </summary>

using System;
using System.IO;
using System.Collections;
using NUnit.Framework;

using TopCoder.File.Statistics.Matcher;
using TopCoder.File.Statistics.Reporting;
using TopCoder.File.Statistics.Processor;
using TopCoder.File.Statistics.Processor.LineCounter;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.FailureTests.Reporting
{
	/// <summary>
    /// This test fixture contains the failure tests for <see cref="BasicStatsReporting"/> class.
    ///
    /// <author>MPhk</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    [TestFixture]
	public class BasicStatsReportingTests {
	
    	/// <summary>
	    /// The parent base path of the test files.
	    /// </summary>
	    private const string PARENT_BASE_PATH = "../../test_files/failuretests";
    	
	    /// <summary>
	    /// The the base path of the test files.
	    /// </summary>
	    private const string BASE_PATH = "../../test_files/failuretests/reporting";
	
	    /// <summary>
	    /// A BasicStatsReporting instance to test.
	    /// </summary>
	    private BasicStatsReporting reporting = null;
	
	    /// <summary>
	    /// A StatsResult instance to test.
	    /// </summary>
	    private StatsResult result = null;
	
	    /// <summary>
	    /// A FileProcessor instance to test.
	    /// </summary>
	    private TextLineCounter processor;
	
	    /// <summary>
	    /// Sets up this test.
	    /// </summary>
	    [SetUp]
	    protected void SetUp() {
	        reporting = new BasicStatsReporting();
	        result = new StatsResult();
	        processor = new TextLineCounter("text", new ExtensionMatcher("txt"));
	    }
	
	    /// <summary>
	    /// Checks whether the constructor BasicStatsReporting(string, string) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestBasicStatsReporting1() {
            new BasicStatsReporting(null, "prefix.");
	    }
	    
	    /// <summary>
	    /// Checks whether the constructor BasicStatsReporting(string, string) properly throws
	    /// ArgumentException for empty argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestBasicStatsReporting2() {
            new BasicStatsReporting("", "prefix.");
	    }
	    
	    /// <summary>
	    /// Checks whether the constructor BasicStatsReporting(string, string) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestBasicStatsReporting3() {
            new BasicStatsReporting("namespace", null);
	    }
	
	    /// <summary>
	    /// Checks whether the method aggregate(FileProcessor, File, StatsResult) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestAggregate1() {
        	reporting.Aggregate(null, new FileInfo("file"), new StatsResult());
	    }
	
	    /// <summary>
	    /// Checks whether the method aggregate(FileProcessor, File, StatsResult) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestAggregate2() {
            reporting.Aggregate(processor, null, result);
	    }
	
	    /// <summary>
	    /// Checks whether the method aggregate(FileProcessor, File, StatsResult) properly throws
	    /// ArgumentNullException for null argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentNullException))]
	    public void TestAggregate3() {
            reporting.Aggregate(processor, new FileInfo("file"), null);
	    }
	
	    /// <summary>
	    /// Checks whether the method aggregate(FileProcessor, File, StatsResult) properly throws
	    /// StatsReportingException for invalid file argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(StatsReportingException))]
	    public void TestAggregate4() {
	        reporting.Reset(new DirectoryInfo(BASE_PATH));
            reporting.Aggregate(processor, new FileInfo(BASE_PATH), result);
	    }
	
	    /// <summary>
	    /// Checks whether the method aggregate(FileProcessor, File, StatsResult) properly throws
	    /// StatsReportingException for invalid file argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(StatsReportingException))]
	    public void TestAggregate5() {
	        reporting.Reset(new DirectoryInfo(BASE_PATH));
            reporting.Aggregate(processor, new FileInfo(BASE_PATH + "invalid_file"), result);
	    }
	
	    /// <summary>
	    /// Checks whether the method aggregate(FileProcessor, File, StatsResult) properly throws
	    /// StatsReportingException for invalid file argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(StatsReportingException))]
	    public void TestAggregate6() {
	        reporting.Reset(new DirectoryInfo(BASE_PATH));
            reporting.Aggregate(processor, new FileInfo(new DirectoryInfo(PARENT_BASE_PATH) + "/file.txt"), result);
	    }
	
	    /// <summary>
	    /// Checks whether the method reset(File) properly throws
	    /// ArgumentException for invalid folder argument.
	    /// </summary>
	    [Test, ExpectedException(typeof(ArgumentException))]
	    public void TestReset() {
            DirectoryInfo baseDir = new DirectoryInfo("INVALID_DIR");
            reporting.Reset(baseDir);
	    }
    }
}
