/*
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0b
 */

using System;
using System.Collections;
using System.Reflection;
using TopCoder.Util.ConfigurationManager;
using NUnit.Framework;
using System.IO;

namespace TopCoder.File.Statistics.Processor.LineCounter
{
    /// <summary>
    /// This class is a base for verifying basic functionality of <c>FileProcessorBase</c> implementations, based on
    /// <c>TestContract</c>.
    /// </summary>
    [TestFixture]
    public class TestTextLineCounter : TestFileProcessorBase
    {
        /// <summary>
        /// Sets up the processors used in testing.
        /// </summary>
        [SetUp]
        public override void Setup()
        {
            type = typeof(TextLineCounter);
            base.Setup();
        }

        /// <summary>
        /// Tests the direct (alias, matcher) constructor with null parameters.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestNullDirectConstructor2()
        {
            new TextLineCounter("alias", (IFileMatcher) null);
        }

        /// <summary>
        /// Tests null parameters for the contract constructor. Overriden due to ambiguity.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public override void TestContractConstructorNull2()
        {
            new TextLineCounter("namespace", (string) null);
        }

        /// <summary>
        /// Tests the processing method for a few text files, comparing against precomputed correct values.
        /// 
        /// This method tests where blank lines ARE ignored.
        /// </summary>
        [Test]
        public void TestProcess1()
        {
            Assert.AreEqual(8, CountLines(2, @"..\..\test_files\TestFile1.cs"), "2 - TestFile1.cs");
            Assert.AreEqual(16, CountLines(2, @"..\..\test_files\TestFile2.cs"), "2 - TestFile2.cs");
            
            Assert.AreEqual(8, CountLines(4, @"..\..\test_files\TestFile1.cs"), "4 - TestFile1.cs");
            Assert.AreEqual(16, CountLines(4, @"..\..\test_files\TestFile2.cs"), "4 - TestFile2.cs");
        }
        
        /// <summary>
        /// Tests the processing method for a few text files, comparing against precomputed correct values.
        /// 
        /// This method tests where blank lines ARE NOT ignored.
        /// </summary>
        [Test]
        public void TestProcess2()
        {
            Assert.AreEqual(10, CountLines(0, @"..\..\test_files\TestFile1.cs"), "0 - TestFile1.cs");
            Assert.AreEqual(19, CountLines(0, @"..\..\test_files\TestFile2.cs"), "0 - TestFile2.cs");
            
            Assert.AreEqual(10, CountLines(1, @"..\..\test_files\TestFile1.cs"), "1 - TestFile1.cs");
            Assert.AreEqual(19, CountLines(1, @"..\..\test_files\TestFile2.cs"), "1 - TestFile2.cs");
            
            Assert.AreEqual(10, CountLines(3, @"..\..\test_files\TestFile1.cs"), "3 - TestFile1.cs");
            Assert.AreEqual(19, CountLines(3, @"..\..\test_files\TestFile2.cs"), "3 - TestFile2.cs");
        }
        
        /// <summary>
        /// Tests the processing method for a few text files, comparing against precomputed correct values.
        /// 
        /// This method tests where blank lines ARE ignored, by simulating a command line call.
        /// </summary>
        [Test]
        public void TestHandleArguments1()
        {
            Hashtable arguments = new Hashtable();
            arguments[TextLineCounter.IgnoreBlankLinePropertyKey] = "";
            ((IArgumentHandler) procs[0]).HandleArguments(arguments);
            
            Assert.AreEqual(8, CountLines(0, @"..\..\test_files\TestFile1.cs"), "0 - TestFile1.cs");
            Assert.AreEqual(16, CountLines(0, @"..\..\test_files\TestFile2.cs"), "0 - TestFile2.cs");
        }
        
        /// <summary>
        /// Tests the processing method for a few text files, comparing against precomputed correct values.
        /// 
        /// This method tests where blank lines ARE NOT ignored, while simulating a command line call.
        /// </summary>
        [Test]
        public void TestHandleArguments2()
        {
            Hashtable arguments = new Hashtable();
            ((IArgumentHandler) procs[0]).HandleArguments(arguments);
            
            Assert.AreEqual(10, CountLines(0, @"..\..\test_files\TestFile1.cs"), "0 - TestFile1.cs");
            Assert.AreEqual(19, CountLines(0, @"..\..\test_files\TestFile2.cs"), "0 - TestFile2.cs");
        }
        
        /// <summary>
        /// This method tests a null argument.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestHandleArgumentsNull()
        {
            ((IArgumentHandler) procs[0]).HandleArguments(null);
        }
        
        /// <summary>
        /// This method tests a non-string value in the hashtable.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestHandleArgumentsNonString()
        {
            Hashtable arguments = new Hashtable();
            arguments[TextLineCounter.IgnoreBlankLinePropertyKey] = 42;
            ((IArgumentHandler) procs[0]).HandleArguments(arguments);
        }
        
        /// <summary>
        /// Tests the option provider.
        /// </summary>
        [Test]
        public void TestProposeOptions()
        {
            IList options = ((IArgumentHandler) procs[0]).ProposeOptions();
            
            Assert.AreEqual(1, options.Count, "Exactly one option expected.");
            Assert.AreEqual(TextLineCounter.IgnoreBlankLinePropertyKey, ((ArgumentOption) options[0]).SwitchName,
                            "Switch name incorrect.");
        }
    }
}
